/*
Apollon @as-integrations/aws-lambda -apukirjaston avulla graphQL-serveristä 
voidaan luoda laukaistava lambda-funktio
*/
const { ApolloServer } = require('@apollo/server');
const {
  startServerAndCreateLambdaHandler,
  handlers,
} = require('@as-integrations/aws-lambda');

// Tietokantaa käsittelevät funktiot
const listBooks = require('./ddbfunctions/listbooks');
const putBook = require('./ddbfunctions/putbook');

// skeema menee typeDefs-muuttujaan
const typeDefs = `#graphql
type Book {
  id: ID!
  bookname: String!
}
input BookInput {
  bookname: String!
}
type Query {
  listBooks: [Book]
}
type Mutation {
  putBook(input: BookInput): Book
}
schema {
  query: Query
  mutation: Mutation
}
`;

/* resolverit yhdistävät kyselyt ja mutaatiot ne toteuttaviin funktioihin
   Huomaa että vasemmalla ovat skeeman termit ja oikealla funktioiden nimet,
   Nämä ovat eri asioita, vaikka nimet ovatkin samat.
*/

const resolvers = {
  Query: {
    listBooks: listBooks,
  },
  Mutation: {
    putBook: putBook,
  },
};

const server = new ApolloServer({ typeDefs, resolvers });

// tehdään serveristä lambda-funktio, jonka laukaisu käynnistää serverin
exports.handler = startServerAndCreateLambdaHandler(
  server,
  handlers.createAPIGatewayProxyEventV2RequestHandler()
);